#include "zeusmp.def"

C Asif ud-Doula 11/15/2010
C modified for f77 zeusmpv1.5
C jcvernaleo 11/2011
C       A simple stellar wind with constant flow veocity
C       and density at the base, no forces, but include
c       rotation and split dipole magnetic field.
C       The purpose is to verify angular momentum problem 
C       at the outer i-boundary wherein v_phi builds up
C       for supersonic flows.

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                W I N D                    \\\\\\\\\\
c
c
c=======================================================================
c
       subroutine wind

       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"

       integer i,j,k,km1,jm1
       REAL rho_b,vinf,rsta,vrot,Bstar,ome,rad
c       base density
        rho_b=1.e-10
        vinf=2.e8
        rsta=5.e11
        vrot = 3.e7
        Bstar=1.e3
        ome=vrot/rsta
        print *, 'rho_b', rho_b

       write(6,"(10x,'MYPROB  : Initialisation commencing')")
c
c
      do k=ks-2,ke+3
         do j=js-2,je+3
            do i=is-2,ie+3
c
c
c               print *, x1a(i)
               d(i,j,k)=rho_b*rsta**2/x1a(i)**2
c               if(i.eq.3) print *, d(i,j,k), rsta**2,rho_b
               e(i,j,k)=d(i,j,k)*ciso*ciso
               v1(i,j,k)=vinf
               v2(i,j,k)=0.e0
               v3(i,j,k)=0.e0
c       try split monopole
          rad=max(rsta,x1a(i))
          b1(i,j,k) = Bstar*cos(x2b(j))*x1a(is)**2/rad**2
c          b2(i,j,k) = .5*Bstar*sin(x2a(j))*x1a(is)**3/x1b(i)**3
          b2(i,j,k) =0.e0
          b3(i,j,k) =0.e0

            enddo
         enddo
      enddo
c       input boundary values
       if(coords(1) .eq.0) then ! set inner boundary
       do j=js,je
         do k=ks,ke
         niib(j,k)=3
         niib2(j,k)=3
         niib3(j,k)=3 
         niib23 (j,k)=3
       enddo
       enddo

       do j=js-2,je+3
         do k=ks-2,ke+3

         diib(j,k,1)=rho_b
         diib(j,k,2)=rho_b
c         eiib(j,k,1)=rho_b*ciso*ciso
c         eiib(j,k,2)=rho_b*ciso*ciso
         v1iib(j,k,1)=vinf  
         v1iib(j,k,2)=vinf  
         v2iib(j,k,1)=0.0
         v2iib(j,k,2)=0.0
         v3iib(j,k,1)=ome*x1b(is-1)*abs(sin(x2b(j)))
         v3iib(j,k,2)=ome*x1b(is-1)*abs(sin(x2b(j)))

         enddo
       enddo
c       ensure there are no emf's along axes
       do j=js-2,je+3
         do k=ks-2, ke+3
        emf3iib(j,k,1)= 0.d0
        emf3iib(j,k,2)= 0.d0
        emf3iib(j,k,3)= 0.d0
c       for models with rotation, try to set emf1,emf2 to zero
        emf1iib(j,k,1) = 0.d0
        emf1iib(j,k,2) = 0.d0
        emf2iib(j,k,1) = 0.d0
        emf2iib(j,k,2) = 0.d0
         enddo
        enddo
       do j=js-2,je+3
         do k=ks-2, ke+3
c       define B field boundary conditions
         km1=max(1,k-1)
         jm1=max(1,j-1)
          b1iib(j,k,1)=b1(is,j,k)
          b1iib(j,k,2)=b1(is-1,j,k) !b1(is-1,j,k)
          b2iib(j,k,1)= b2(is-1,j,k)
          b2iib(j,k,2)=b2(is-2,j,k)  !b2(is-2,j,k)
          b3iib(j,k,1)=b3(is-1,j,k)
          b3iib(j,k,2)=b3(is-2,j,k) !b3(is-2,j,k)
         enddo
       enddo
       do j=js,je+1
        do k=ks,ke+1
         km1=max(1,k-1)
         jm1=max(1,j-1)
c set emf's'
           emf1iib(j,k,1) = ( ( v2iib(j,k,1) + v2iib(j ,km1,1) )
     1 * ( b3iib(j,k,1) + b3iib(jm1,k,1) )
     2 - ( v3iib(j,k,1) + v3iib(jm1,k,1 ) )
     3 * ( b2iib(j,k,1)+b2iib(j,km1,1) ) )*0.25
c
           emf1iib(j,k,2) = ( ( v2iib(j,k,2) + v2iib(j ,km1,2) )
     1 * ( b3iib(j,k,2) + b3iib(jm1,k,2) )
     2 - ( v3iib(j,k,2) + v3iib(jm1,k,2 ) )
     3 * ( b2iib(j,k,2)+b2iib(j,km1,2) ) )*0.25
        enddo
       enddo  !emf1: see bvalemf.F for boundary ranges

        do j=js-2,je+2
         do k=ks-2,ke+3
         km1=max(1,k-1)
         jm1=max(1,j-1)
           emf2iib(j,k,1) = ( v3iib(j,k,1)
     1 * ( b1iib(j,k,1) + b1iib(j,km1,1) )
     2 - ( v1iib(j,k,1) + v1iib(j,km1,1) )
     3 * b3iib(j,k,1) ) * 0.5

c
           emf2iib(j,k,2) = ( v3iib(j,k,2)
     1 * ( b1iib(j,k,2) + b1iib(j,km1,2) )
     2 - ( v1iib(j,k,2) + v1iib(j,km1,2) )
     3 * b3iib(j,k,2) ) * 0.5

         enddo
        enddo !emf2

        do j=js,je+1
         do k=ks-2,ke+2

         km1=max(1,k-1)
         jm1=max(1,j-1)
           emf3iib(j,k,1) = ( ( v1iib(j,k,1) + v1iib(jm1,k,1) )
     1 * b2iib(j,k,1)
     2 - v2iib(j,k,1)
     3 * ( b1iib(j,k,1)+b1iib(jm1,k,1)))*0.5
c
           emf3iib(j,k,2) = ( ( v1iib(j,k,2)+v1iib(jm1,k,2) )
     1 * b2iib(j,k,2)
     2 - v2iib(j,k,2)
     3 * ( b1iib(j,k,2)+b1iib(jm1,k,2) ) ) * 0.5
c
           emf3iib(j,k,3) = 2.0 * emf3iib(j,k,2) - emf3iib(j,k,1)
         enddo
        enddo

      endif !coord(1)
       if(coords(2) .eq.0 ) then ! set j boundary
        do k=ks-2,ke+3
          do i=is-2,ie+3
             nijb(i,k)=-1
          enddo
        enddo
      endif! coords(2)


       if( coords(2).eq.(ntiles(2)-1)) then ! set inner boundary
        do k=ks-2,ke+3
          do i=is-2,ie+3
             nojb(i,k)=-1
          enddo
        enddo
      endif! coords(2)


      do i = 1, 6
       bvstat(i,1) = 0
       bvstat(i,2) = 0
       bvstat(i,3) = 0
       bvstat(i,4) = 0
       bvstat(i,5) = 0
       bvstat(i,6) = 0
       bvstat(i,7) = 0
       bvstat(i,8) = 0
      enddo
      print *, 'density', d(3,3,3), 'vel', v1(3,3,3)

c       call bndyflgs
c        call bvalemf1
c        call bvalemf2
c        call bvalemf3


       return
       end

