#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\        B E G I N   P R O G R A M          //////////
c    //////////               Z E U S M P                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
      program zeusmp
c
c PURPOSE
c   Main program for 3-D MPI version of ZEUS.
c
c AUTHOR
c   Robert A. Fiedler
c
c LAST MODIFIED by John C. Vernaleo
c   04/30/2007
c.......................................................................
c
c DECLARATIONS
c
      implicit NONE
#include "mod_names.h"
#include "options.h"
#include "param.h"
#include "field.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "clocks.h"
#include "impsoln.h"
#include "gravity.h"
#include "radiation.h"
c
      REAL     zcs
      integer i,j,k

CJH
      REAL t1, t2

#ifdef UNICOS
      real       cpuall
#else
      real*8     cpuall
#endif /* UNICOS */

#ifdef GRAV_FFT
      integer nx,ny,nz,snz,maxn
      external create_plan_
      common /fftw/ maxn
#endif /* GRAV_FFT */
      external mstart,dataio,srcstep,nudt,newgrid,intchk,SPECIAL
c
c DATA
c
      data nhy, time, nsub / 0, 0.0, 0 /
      data zcs / 0. /
#ifndef SHELLS
      data ifsen / 0, 0, 1, 1, 1, 1/
#else
      data ifsen / 0, 0, 1, 1, 1, 1, 1/
#endif /* NOT SHELLS */
      data myid_w, myid, nprocs_w, nprocs / 0, 0, 1, 1 /
      data coords / 0, 0, 0 /
      data reorder  / .true. /

#ifdef MPI_USED
c
c Initialize MPI
c
      call MPI_INIT( ierr )
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid_w  , ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, nprocs_w, ierr )
#endif /* MPI_USED */
c
c  Master writes greeting.
c
#ifdef MPI_USED
      if (myid_w .eq. 0) then
#endif /* MPI_USED */
        write(6,"(///10x,'ZZZZZ EEEEE U   U  SSSS     M   M PPPP ')")
        write(6,   "(10x,'   Z  E     U   U S         MM MM P   P')")
        write(6,   "(10x,'  Z   EEEE  U   U  SSS  === M M M PPPP ')")
        write(6,   "(10x,' Z    E     U   U     S     M   M P    ')")
        write(6,   "(10x,'ZZZZZ EEEEE  UUU  SSSS      M   M P    ')")
        write(6,"()")
        write(6,"()")
        write(6,"(10x,'       DEVELOPED BY ROBERT A. FIEDLER')")
        write(6,"(10x,'       Maintained by John C. Vernaleo')")
        write(6,"(10x,'         ZEUS-MP V1.5.18 - 12/31/2011')")
        write(6,"()")
        write(6,"(10x,'RUNNING AS ',i4,' PROCESS(ES)')") nprocs_w
        write(6,"(10x,'WITH THE FOLLOWING CONFIGURATION:')")
        write(6,"()")
        write(6, GEOMETRY_OPTION  )
        write(6, GRID_OPTION  )
        write(6, PT_MASS_OPTION   )
        write(6, GRAVITY_OPTION   )
        write(6, SELF_GRAVITY_OPTION   )
#ifdef GRAV_FFT
        write(6, PS_OPTION   )
#endif
        write(6, MHD_OPTION       )
        write(6, RAD_IMP_OPTION )
        write(6, RAD_EXP_OPTION )
        write(6, PRECISION_OPTION )
        write(6, CPU_TIMER_OPTION )
        write(6, PROFILING_OPTION )
        write(6, MPI_OPTION )
        write(6, MPE_OPTION )
        write(6, DEBUG_OPTION )
        write(6, RES_DUMPS_OPTION )
        write(6, HDF_DUMPS_OPTION )
        write(6, TSL_DUMPS_OPTION )
        write(6, HST_DUMPS_OPTION )
        write(6, TXT_DUMPS_OPTION )
        write(6, FORCE_OPTION )
        write(6, ALFLIMIT_OPTION )
        write(6, RAD_LOSS_OPTION )
        write(6, VISC_OPTION )
        write(6, CONDUCT_OPTION )
        write(6,"()")
#ifdef MPI_USED
      endif
#endif /* MPI_USED */
c
c Set up the problem: read input deck and possibly a restart file.
c
      call mstart
c
c Write out initial data dumps.
c
c      call dataio( iswres, iswhdf, iswhst, iswusr 
#ifdef SHELLS
c    .              iswshl
#endif /* SHELLS */
c    .                                             )
      call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6)
#ifdef SHELLS
     .            ,ifsen(7)
#endif /* SHELLS */
     .                                                   )
c
c  Initialize cpu and wall clocks.  The quantities "cputime" and
c  "wclock" are the CPU and wall clock times (in seconds) in the main 
c  loop.
c
        wclock0 = 0.0
        cputime0 = 0.0
        call clocks (cputime, wclock)
        wclock0 = wclock
        cputime0 = cputime

      if (myid .eq. 0)then
         write(6,"(/,' Set-up complete with ',i2,' warning(s):'
     &        ,' entering main loop...')") nwarn
      endif

CPS
#ifdef GRAV_FFT
C
C Create best calculating plans for FFTW to perform forward
C and backward FFT.
C
      nx=ie-is+1
      ny=je-js+1
      nz=ke-ks+1
      maxn=nx
      if(maxn.LT.ny) maxn=ny
      if(maxn.LT.nz) maxn=nz

      if(ntiles(1).GT.1) call create_plan(nx*nprocs_w,ny,nz)
      if(ntiles(2).GT.1) call create_plan(nx,ny*nprocs_w,nz)
      if(ntiles(3).GT.1) call create_plan(nx,ny,nz*nprocs_w)

#endif /* GRAV_FFT */
c
c--------------------------  start of main loop  -----------------------
c
c Execution ends when INTCHK returns a value of 1 for ifsen(1).
c
#ifdef RAD
cjh   initial N-R and lssolve iteration counters for implicit radiation
cjh   module
      totlsit = 0.D0
      totnrit = 0.D0
#endif /* RAD */
1000  continue

      nsub = 1

CPS
#if defined GRAV && defined GRAV_FFT
c
c Solve Poisson's equation for the gravitational potential.
c
c
c      turn off self-gravity (CSR)
c      turned back on, but in an ifdef now (jcv)
      if (time .gt. tgrav) call gravity
#endif /* GRAV && GRAV_FFT */
C
c
c Call optional boundary update step
c 10/18/03 jcv
c based on similar thing from zeus3d
c
        call BNDYUPDATE
c
c Evaluate all non-advective terms in the evolution equations.
c
        call srcstep
c
c Compute the advection of all field variables.
c
        call transprt
c
c Optional call user-supplied module.
c
        call SPECIAL
c
c Update the step counter and current time value.
c
        nhy   = nhy   + 1
        time  = time  + dt
c
c Check the CPU time, number of steps, output times to determine if
c a stopping criterion has been met or output is desired.
c Also check for keyboard input, depending on the value of mbatch.
c
c        call intchk( iswres, iswhdf, iswhst, iswusr )
        call intchk( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6) )
c
c Compute new timestep
c
c        call nantest
        call nudt
#ifdef MOVING_GRID
c
c Update the grid and related quantites.
c
        call newgrid
#endif
c
c Write out any desired output files now that everything has been
c updated.
c Skip dataio if the run is being terminated to avoid duplicate output.
c
        if (ifsen(1) .eq. 1) goto 2000
        call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6)
#ifdef SHELLS
     .              ,ifsen(7)
#endif /* SHELLS */
     .                                                     )
c
      goto 1000  !  Loop back to begin the next time step.
c
c--------------------------  end of main loop  -------------------------
c
c Terminate the run by making final dumps, write goodbyes
c
2000  continue
      call clocks (cputime, wclock)
#ifdef NO_CPU_TIMER
      tused = real(wclock)
#else
      tused = real(cputime)
#endif /* NO_CPU_TIMER */
      ifsen(2) = 1
      ifsen(3) = 1
      ifsen(4) = 1
      ifsen(5) = 1
      ifsen(6) = 1
#ifdef SHELLS
      ifsen(7) = 1
#endif /* SHELLS */
      call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5) , ifsen(6)
#ifdef SHELLS
     .            ,ifsen(7)
#endif /* SHELLS */
     .                                                   )
#ifdef MPI_USED
c
c Sum everyone's cputime (stored in tused) to get CPU time for all 
c processes.
c
      call MPI_REDUCE(tused, cpuall, 1, MPI_FLOAT,
     &                MPI_SUM, 0, comm3d, ierr )
#else /* MPI_USED */
      cpuall = tused
#endif /* MPI_USED */
      if (myid .eq. 0) then      
c
c Let's assume tused is user + system time on the master thread.
c One would also like to keep track of wall-clock time and the sum
c of CPU times used by each processor.
c
        zcs = real(nprocs_w*nhy*nx1z*nx2z*nx3z)/(tused+tiny)
        write(6,"(/' Execution terminated with ',i4,' warning(s)')") 
     &     nwarn
        write(6,"(/' Performance summary:')")
        write(6,"('  zone-cycles per cpu second =',1pe12.5)") zcs
        write(6,"('  Master CPU                 =',1pe12.5, ' sec')") 
     &     tused
        write(6,"('  Average CPU/node           =',1pe12.5, ' sec')") 
     &     cpuall/real(nprocs_w)
        write(6,"('  Wall Clock                 =',1pe12.5, ' sec')") 
     &     wclock
        write(6,"()")
        close(unit=2)
        close(unit=3)
        close(unit=30)
#ifdef TSL_DUMPS
        close(unit=31)
#endif /* TSL_DUMPS */
      endif

#ifdef RAD
cjh   write out implicit radiation iteration summary
#ifdef MPI_USED
      if(myid_w .eq. 0) then
#endif /* MPI_USED */
         nrpert = totnrit / nhy
         lspert = totlsit / nhy
         print *,'Average LSSOLVE iter. per tstep: ',lspert
         print *,'Average imp. rad. NR iter. per tstep: ',nrpert
#ifdef MPI_USED
      endif
#endif /* MPI_USED */
#endif /* RAD */
#ifdef MPI_USED
c
c Turn off MPI
c
      call MPI_FINALIZE ( ierr )
#endif /* MPI_USED */
c
c=======================================================================
c
c    \\\\\\\\\\          E N D  P R O G R A M             //////////
c    //////////               Z E U S M P                 \\\\\\\\\\
c
c=======================================================================
c
      end
